﻿Class DrawFreehandLine : Implements Api.Invoker, ICloneable
    Const _class = "DrawFreehandLine"

    Dim _canvas As Api.Canvas
    Dim _tablet As Tablet
    Dim _strokePointPair As Api.StrokePointPair = Nothing
    Dim _strokePointPairs As List(Of Api.StrokePointPair) = New List(Of Api.StrokePointPair)
    Dim _strokePointPairsIndex As Long = -1
    Dim _image As Image
    Dim _lock = New Threading.ReaderWriterLockSlim

    ReadOnly Property StrokePointPair() As Api.StrokePointPair Implements Api.Invoker.StrokePointPair
        Get
            StrokePointPair = Me._strokePointPair
        End Get
    End Property

    Property StrokePointPairs() As List(Of Api.StrokePointPair) Implements Api.Invoker.StrokePointPairs
        Get
            StrokePointPairs = Me._strokePointPairs
        End Get
        Set(ByVal value As List(Of Api.StrokePointPair))
            Me._strokePointPairs = value
        End Set
    End Property

    Sub AddStrokePointPair(ByVal strokePointPair As Api.StrokePointPair) Implements Api.Invoker.AddStrokePointPair
        Me._strokePointPairs.Add(strokePointPair)
    End Sub

    Property Canvas() As Api.Canvas Implements Api.Invoker.Canvas
        Get
            Canvas = Me._canvas
        End Get
        Set(ByVal value As Api.Canvas)
            Me._canvas = value
        End Set
    End Property

    Property StrokeModifier() As Api.StrokeModifier Implements Api.Invoker.StrokeModifier
        Get
            StrokeModifier = Me._tablet.StrokeModifier(DrawFreehandLine._class)
        End Get
        Set(ByVal value As Api.StrokeModifier)
            Me._tablet.StrokeModifier(DrawFreehandLine._class) = value
        End Set
    End Property

    Private Sub Invoke() Implements Api.Invoker.Invoke
        With Me._Lock
            .EnterWriteLock()
            Me._strokePointPairsIndex = Me._strokePointPairsIndex + 1
            Me._strokePointPair = Me._strokePointPairs(Me._strokePointPairsIndex)
            Debug.Assert(Me._strokePointPair IsNot Nothing, "Invoke()", "Count:" & Me._strokePointPairs.Count & ", Index:" & Me._strokePointPairsIndex)
            Dim command As IEnumerator(Of Api.Command) = Me._tablet.Commands(DrawFreehandLine._class).GetEnumerator()
            Do While command.MoveNext()
                command.Current.Invoke(Me)
            Loop
            'Log.add(Api.Invoker)
            .ExitWriteLock()
        End With
    End Sub

    Public Function Clone() As Object Implements System.ICloneable.Clone
        Clone = Me.MemberwiseClone()
        Debug.Assert(Equals(Clone, Me) AndAlso Not ReferenceEquals(Clone, Me))
    End Function

    Sub New(ByVal tablet As Tablet)
        Try
            Dim obj = DirectCast(Me, ICloneable)
        Catch ex As Exception
            Debug.Fail(ex.Message)
        End Try
        Me._tablet = tablet
    End Sub
End Class
